package dev.micle.firefly_bush_backport.particle;

import dev.micle.firefly_bush_backport.config.Config;
import net.minecraft.client.multiplayer.ClientLevel;
import net.minecraft.client.particle.*;
import net.minecraft.core.BlockPos;
import net.minecraft.core.particles.SimpleParticleType;
import net.minecraft.util.Mth;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;
import org.jetbrains.annotations.NotNull;

import javax.annotation.ParametersAreNonnullByDefault;
import java.awt.*;
import java.util.Optional;

@OnlyIn(Dist.CLIENT)
public class FireflyParticle extends TextureSheetParticle {
    protected FireflyParticle(ClientLevel clientLevel, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
        super(clientLevel, x, y, z, xSpeed, ySpeed, zSpeed);
        this.speedUpWhenYMotionIsBlocked = true;
        this.friction = Config.Client.particleExtraFriction.get().floatValue();
        this.quadSize *= Config.Client.particleExtraScale.get().floatValue();
        this.yd *= 0.8F;
        this.xd *= 0.8F;
        this.zd *= 0.8F;
    }

    @Override
    public @NotNull ParticleRenderType getRenderType() {
        return ParticleRenderType.PARTICLE_SHEET_TRANSLUCENT;
    }

    @Override
    public int getLightColor(float partialTick) {
        return (int) (Config.Client.particleExtraBrightness.get().floatValue() * getFadeAmount(
                this.getLifetimeProgress(this.age + partialTick),
                Config.Client.particleFadeInLightTime.get().floatValue(),
                Config.Client.particleFadeOutLightTime.get().floatValue())
        );
    }

    private static float getFadeAmount(float lifetimeProgress, float fadeIn, float fadeOut) {
        if (lifetimeProgress >= 1.0F - fadeIn) {
            return (1.0F - lifetimeProgress) / fadeIn;
        } else {
            return lifetimeProgress <= fadeOut ? lifetimeProgress / fadeOut : 1.0F;
        }
    }

    private float getLifetimeProgress(float age) {
        return Mth.clamp(age / this.lifetime, 0.0F, 1.0F);
    }

    @Override
    public void tick() {
        super.tick();
        if (!this.level.getBlockState(BlockPos.containing(this.x, this.y, this.z)).isAir()) {
            this.remove();
        } else {
            this.setAlpha(getFadeAmount(this.getLifetimeProgress(this.age),
                    Config.Client.particleFadeInAlphaTime.get().floatValue(),
                    Config.Client.particleFadeOutAlphaTime.get().floatValue())
            );
            if (Math.random() > 0.95 || this.age == 1) {
                this.setParticleSpeed(-0.05F + 0.1F * Math.random(), -0.05F + 0.1F * Math.random(), -0.05F + 0.1F * Math.random());
            }
        }
    }

    @OnlyIn(Dist.CLIENT)
    public static class FireflyProvider implements ParticleProvider<SimpleParticleType> {
        private final SpriteSet sprite;

        public FireflyProvider(SpriteSet spriteSet) {
            this.sprite = spriteSet;
        }

        @ParametersAreNonnullByDefault
        public Particle createParticle(
                SimpleParticleType simpleParticleType,
                ClientLevel clientLevel,
                double x,
                double y,
                double z,
                double xSpeed,
                double ySpeed,
                double zSpeed
        ) {
            FireflyParticle fireflyParticle = new FireflyParticle(
                    clientLevel,
                    x,
                    y,
                    z,
                    0.5 - clientLevel.random.nextDouble(),
                    clientLevel.random.nextBoolean() ? ySpeed : -ySpeed,
                    0.5 - clientLevel.random.nextDouble()
            );
            fireflyParticle.setLifetime(clientLevel.random.nextIntBetweenInclusive(
                    Config.Client.particleMinLifetime.get(), Config.Client.particleMaxLifetime.get()
            ));
            fireflyParticle.scale(1.5F);
            fireflyParticle.pickSprite(this.sprite);
            fireflyParticle.setAlpha(0.0F);

            Optional<Color> optionalColor = Config.Client.particleExtraColorsWeightedList.getRandomValue(clientLevel.random);
            optionalColor.ifPresent(color ->
                    fireflyParticle.setColor(color.getRed() / 255F, color.getGreen() / 255F, color.getBlue() / 255F)
            );

            return fireflyParticle;
        }
    }
}
